@@ -3,19 +3,29 @@ import time
 import pkg_resources
 import os
 
-from typing import Dict, Any, TypeVar, Type, List, Tuple, cast
+from typing import Dict, Any, List, Tuple, Optional, cast
 
 import click
 import json
 from jinja2 import Environment, BaseLoader
 
-from pydantic import parse_obj_as, ValidationError
 from gordo import __version__
 from gordo.workflow.config_elements.normalized_config import NormalizedConfig
 from gordo.workflow.workflow_generator import workflow_generator as wg
+from gordo.workflow.config_elements.schemas import (
+    SecurityContext,
+    PodSecurityContext,
+    EnvVar,
+)
 from gordo.cli.exceptions_reporter import ReportLevel
 from gordo.util.version import parse_version
-from gordo.dependencies import configure_once
+from gordo.workflow.workflow_generator.helpers import (
+    parse_argo_version,
+    determine_argo_version,
+    ArgoVersionError,
+)
+from gordo_core.back_compatibles import DEFAULT_BACK_COMPATIBLES
+from .custom_types import JSONParam, REParam
 
 
 logger = logging.getLogger(__name__)
@@ -52,43 +62,7 @@ def get_builder_exceptions_report_level(config: NormalizedConfig) -> ReportLevel
     return report_level
 
 
-T = TypeVar("T")
-
-
-def parse_json(value: str, schema: Type[T]):
-    try:
-        data = json.loads(value)
-    except json.JSONDecodeError as e:
-        raise click.ClickException('Malformed JSON string: "%s"' % str(e))
-    try:
-        obj = parse_obj_as(schema, data)
-    except ValidationError as e:
-        raise click.ClickException('Schema validation error: "%s"' % str(e))
-    return obj
-
-
-DEFAULT_CUSTOM_MODEL_BUILDER_ENVS = """
-[
-    {
-        "name": "DL_SERVICE_AUTH_STR",
-        "valueFrom": {
-            "secretKeyRef": {
-                "key": "tenant_id_secret",
-                "name": "dlserviceauth"
-            }
-        }
-    },
-    {
-        "name": "DL2_SERVICE_AUTH_STR",
-        "valueFrom": {
-            "secretKeyRef": {
-                "key": "tenant_id_secret",
-                "name": "dl2serviceauth"
-            }
-        }
-    }
-]
-"""
+DEFAULT_CUSTOM_MODEL_BUILDER_ENVS = "[]"
 
 
 def validate_generate_context(context):
@@ -135,6 +109,19 @@ def prepare_resources_labels(value: str) -> List[Tuple[str, Any]]:
     return resources_labels
 
 
+def prepare_argo_version(argo_binary: Optional[str] = None) -> str:
+    if argo_binary is not None:
+        raw_argo_version = determine_argo_version(argo_binary)
+    else:
+        raw_argo_version = determine_argo_version()
+    argo_version = parse_argo_version(raw_argo_version)
+    if argo_version is None:
+        raise ArgoVersionError(
+            "Unable to parse %s version: '%s'" % (argo_binary, argo_version)
+        )
+    return cast(str, argo_version)
+
+
 @click.group("workflow")
 @click.pass_context
 def workflow_cli(gordo_ctx):
@@ -216,14 +203,14 @@ def workflow_cli(gordo_ctx):
 @click.option(
     "--docker-repository",
     type=str,
-    default="gordo",
+    default="equinor",
     help="The docker repo to use for pulling component images from",
     envvar=f"{PREFIX}_DOCKER_REPOSITORY",
 )
 @click.option(
     "--docker-registry",
     type=str,
-    default="auroradevacr.azurecr.io",  # TODO: Change to docker.io after migrating
+    default="ghcr.io",
     help="The docker registry to use for pulling component images from",
     envvar=f"{PREFIX}_DOCKER_REGISTRY",
 )
@@ -294,6 +281,7 @@ def workflow_cli(gordo_ctx):
     help="List of custom environment variables in ",
     envvar=f"{PREFIX}_CUSTOM_MODEL_BUILDER_ENVS",
     default=DEFAULT_CUSTOM_MODEL_BUILDER_ENVS,
+    type=JSONParam(List[EnvVar]),
 )
 @click.option(
     "--prometheus-server-address",
@@ -338,19 +326,77 @@ def workflow_cli(gordo_ctx):
     type=int,
     default=50,
 )
+@click.option(
+    "--gordo-server-readiness-initial-delay",
+    help="initialDelaySeconds for gordo-server's readinessProbe",
+    envvar=f"{PREFIX}_GORDO_SERVER_READINESS_INITIAL_DELAY",
+    type=int,
+    default=5,
+)
+@click.option(
+    "--gordo-server-liveness-initial-delay",
+    help="initialDelaySeconds for gordo-server's livenessProbe",
+    envvar=f"{PREFIX}_GORDO_SERVER_LIVENESS_INITIAL_DELAY",
+    type=int,
+    default=600,
+)
+@click.option(
+    "--security-context",
+    help="Containers securityContext in JSON format",
+    envvar=f"{PREFIX}_SECURITY_CONTEXT",
+    type=JSONParam(SecurityContext),
+)
+@click.option(
+    "--pod-security-context",
+    help="Global Workflow securityContext in JSON format",
+    envvar=f"{PREFIX}_POD_SECURITY_CONTEXT",
+    type=JSONParam(PodSecurityContext),
+)
+@click.option(
+    "--default-data-provider",
+    help="Default data_provider.type for dataset",
+    envvar=f"{PREFIX}_DEFAULT_DATA_PROVIDER",
+)
+@click.option(
+    "--model-builder-class",
+    help="ModelBuilder class",
+    envvar="MODEL_BUILDER_CLASS",
+)
+@click.option(
+    "--argo-binary",
+    default="argo",
+    type=REParam(r"^argo\d*$"),
+    help="Argo binary path",
+)
 @click.pass_context
 def workflow_generator_cli(gordo_ctx, **ctx):
     """
     Machine Configuration to Argo Workflow
     """
 
-    configure_once()
-
     context: Dict[Any, Any] = ctx.copy()
     yaml_content = wg.get_dict_from_yaml(context["machine_config"])
 
+    model_builder_env = None
+    if context["custom_model_builder_envs"]:
+        custom_model_builder_envs = cast(
+            List[EnvVar], context["custom_model_builder_envs"]
+        )
+        model_builder_env = [
+            env_var.dict(exclude_none=True) for env_var in custom_model_builder_envs
+        ]
+    # Create normalized config
+    config = NormalizedConfig(
+        yaml_content,
+        project_name=context["project_name"],
+        model_builder_env=model_builder_env,
+        back_compatibles=DEFAULT_BACK_COMPATIBLES,
+        default_data_provider=context["default_data_provider"],
+        json_path="spec.config",
+    )
+
     try:
-        log_level = yaml_content["globals"]["runtime"]["log_level"]
+        log_level = config.globals["runtime"]["log_level"]
     except KeyError:
         log_level = os.getenv("GORDO_LOG_LEVEL", gordo_ctx.obj["log_level"])
 
@@ -359,17 +405,17 @@ def workflow_generator_cli(gordo_ctx, **ctx):
 
     validate_generate_context(context)
 
+    context["argo_version"] = prepare_argo_version(context.get("argo_binary"))
+
     context["resources_labels"] = prepare_resources_labels(context["resources_labels"])
 
-    model_builder_env = None
-    if context["custom_model_builder_envs"]:
-        model_builder_env = json.loads(context["custom_model_builder_envs"])
-    # Create normalized config
-    config = NormalizedConfig(
-        yaml_content,
-        project_name=context["project_name"],
-        model_builder_env=model_builder_env,
-    )
+    if context["pod_security_context"]:
+        pod_security_context = cast(PodSecurityContext, context["pod_security_context"])
+        context["pod_security_context"] = pod_security_context.dict(exclude_none=True)
+
+    if context["security_context"]:
+        security_context = cast(SecurityContext, context["security_context"])
+        context["security_context"] = security_context.dict(exclude_none=True)
 
     version = parse_version(context["gordo_version"])
     if "image_pull_policy" not in context or not context["image_pull_policy"]:
@@ -405,6 +451,18 @@ def workflow_generator_cli(gordo_ctx, **ctx):
 
     context["builder_runtime"] = builder_runtime
 
+    builder_runtime_env = []
+    if "env" in builder_runtime:
+        builder_runtime_env = builder_runtime["env"]
+
+    if builder_runtime_env:
+        if context["model_builder_class"]:
+            builder_runtime_env.append(
+                {"name": "MODEL_BUILDER_CLASS", "value": context["model_builder_class"]}
+            )
+
+    context["builder_runtime_env"] = builder_runtime_env
+
     context["server_resources"] = config.globals["runtime"]["server"]["resources"]
     context["server_image"] = config.globals["runtime"]["server"]["image"]
 
