@@ -3,14 +3,14 @@
 import typing
 import functools
 import logging
-from typing import Optional, Union, List
+from typing import Optional, Union, List, Iterable, Sized, cast
 from datetime import timedelta, datetime
 
 import numpy as np
 import pandas as pd
 from sklearn.base import TransformerMixin
 
-from gordo.machine.dataset.sensor_tag import SensorTag
+from gordo_core.sensor_tag import SensorTag
 
 logger = logging.getLogger(__name__)
 
@@ -51,7 +51,7 @@ def make_base_dataframe(
     model_input: np.ndarray,
     model_output: np.ndarray,
     target_tag_list: Optional[Union[List[SensorTag], List[str]]] = None,
-    index: typing.Optional[np.ndarray] = None,
+    index: typing.Optional[Union[np.ndarray, pd.Index]] = None,
     frequency: typing.Optional[timedelta] = None,
 ) -> pd.DataFrame:
     """
@@ -69,10 +69,10 @@ def make_base_dataframe(
     model_output: np.ndarray
         Raw model output
     target_tag_list: Optional[Union[List[SensorTag], List[str]]]
-        Tags to be assigned to ``model-output`` if not assinged but model output matches
+        Tags to be assigned to ``model-output`` if not assigned but model output matches
         model input, ``tags`` will be used.
     index: Optional[np.ndarray]
-        The index which should be assinged to the resulting dataframe, will be clipped
+        The index which should be assigned to the resulting dataframe, will be clipped
         to the length of ``model_output``, should the model output less than its input.
     frequency: Optional[datetime.timedelta]
         The spacing of the time between points.
@@ -92,17 +92,23 @@ def make_base_dataframe(
     names_n_values = (("model-input", model_input), ("model-output", model_output))
 
     # Define the index which all series/dataframes will share
-    index = (
-        index[-len(model_output) :] if index is not None else range(len(model_output))
+    normalized_index = (
+        index[-len(model_output) :]
+        if index is not None
+        else list(range(len(model_output)))
     )
 
     # Series to hold the start times for each point or just 'None' values
-    start_series = pd.Series(
-        index
-        if isinstance(index, pd.DatetimeIndex)
-        else (None for _ in range(len(index))),
-        index=index,
-    )
+    series_values: Union[np.ndarray, Iterable]
+    if isinstance(normalized_index, pd.DatetimeIndex):
+        series_values = normalized_index
+    else:
+        series_values = []
+        if (
+            isinstance(normalized_index, pd.Index) and not normalized_index.empty
+        ) or normalized_index:
+            series_values = (None for _ in range(len(cast(Sized, normalized_index))))
+    start_series = pd.Series(series_values, index=normalized_index)
 
     # Calculate the end times if possible, or also all 'None's
     end_series = start_series.map(
@@ -122,7 +128,7 @@ def make_base_dataframe(
     data: pd.DataFrame = pd.DataFrame(
         {("start", ""): start_series, ("end", ""): end_series},
         columns=columns,
-        index=index,
+        index=normalized_index,
     )
 
     # Begin looping over the model-input and model-output; mapping them into
@@ -150,7 +156,9 @@ def make_base_dataframe(
         )
 
         # Pass valudes, offsetting any differences in length compared to index, as set by model-output size
-        other = pd.DataFrame(values[-len(model_output) :], columns=columns, index=index)
+        other = pd.DataFrame(
+            values[-len(model_output) :], columns=columns, index=normalized_index
+        )
         data = data.join(other)
 
     return data
