@@ -1,27 +1,20 @@
 # -*- coding: utf-8 -*-
 
 import logging
-import pydoc
 import copy
 import typing  # noqa
+
 from typing import Union, Dict, Any, Iterable
 from sklearn.pipeline import Pipeline, FeatureUnion
 from sklearn.base import BaseEstimator
 from tensorflow.keras.models import Sequential
-from .utils import validate_import_path
+
+from gordo_core.import_utils import import_location
 
 
 logger = logging.getLogger(__name__)
 
 
-def import_locate(import_path: str) -> Any:
-    obj = pydoc.locate(import_path)
-    if obj is not None:
-        if not validate_import_path(import_path):
-            raise ValueError("Unsupported import path '%s'" % import_path)
-    return obj
-
-
 def from_definition(
     pipe_definition: Union[str, Dict[str, Dict[str, Any]]]
 ) -> Union[FeatureUnion, Pipeline]:
@@ -133,9 +126,12 @@ def _build_step(
 
         import_str = list(step.keys())[0]
 
-        StepClass: Union[FeatureUnion, Pipeline, BaseEstimator] = import_locate(
-            import_str
-        )
+        try:
+            StepClass: Union[
+                None, FeatureUnion, Pipeline, BaseEstimator
+            ] = import_location(import_str)
+        except (ImportError, ValueError):
+            StepClass = None
 
         if StepClass is None:
             raise ImportError(f'Could not locate path: "{import_str}"')
@@ -153,7 +149,10 @@ def _build_step(
         if isinstance(params, dict):
             for param, value in params.items():
                 if isinstance(value, str):
-                    possible_func = import_locate(value)
+                    try:
+                        possible_func = import_location(value)
+                    except (ImportError, ValueError):
+                        possible_func = None
                     if callable(possible_func):
                         params[param] = possible_func
 
@@ -187,7 +186,10 @@ def _build_step(
     # If step is just a string, can initialize it without any params
     # ie. "sklearn.preprocessing.PCA"
     elif isinstance(step, str):
-        Step = import_locate(step)  # type: Union[FeatureUnion, Pipeline, BaseEstimator]
+        try:
+            Step = import_location(step)
+        except (ImportError, ValueError):
+            Step = None
         if hasattr(Step, "from_definition"):
             return getattr(Step, "from_definition")({})
         else:
@@ -210,7 +212,7 @@ def _build_callbacks(definitions: list):
     --------
     >>> callbacks=_build_callbacks([{'tensorflow.keras.callbacks.EarlyStopping': {'monitor': 'val_loss,', 'patience': 10}}])
     >>> type(callbacks[0])
-    <class 'tensorflow.python.keras.callbacks.EarlyStopping'>
+    <class 'keras.callbacks.EarlyStopping'>
 
     Returns
     -------
@@ -264,7 +266,10 @@ def _load_param_classes(params: dict):
 
         # If value is a simple string, try to load the model/class
         if isinstance(value, str):
-            Model: Union[None, BaseEstimator, Pipeline] = import_locate(value)
+            try:
+                Model: Union[None, BaseEstimator, Pipeline] = import_location(value)
+            except (ImportError, ValueError):
+                Model = None
             if Model is not None:
                 if hasattr(Model, "from_definition"):
                     params[key] = getattr(Model, "from_definition")({})
@@ -280,7 +285,10 @@ def _load_param_classes(params: dict):
             and isinstance(value[list(value.keys())[0]], dict)
         ):
             import_path = list(value.keys())[0]
-            Model = import_locate(import_path)
+            try:
+                Model = import_location(import_path)
+            except (ImportError, ValueError):
+                Model = None
 
             sub_params = value[import_path]
 
