@@ -6,12 +6,13 @@ import zlib
 import os
 import io
 import pickle
-import copy
+import re
+import shutil
 
 import dateutil
 import timeit
 from datetime import datetime
-from typing import Union, List, Any
+from typing import Union, List
 
 import pandas as pd
 import pyarrow as pa
@@ -19,10 +20,12 @@ import pyarrow.parquet as pq
 from flask import request, g, jsonify, make_response, Response
 from functools import lru_cache, wraps
 from sklearn.base import BaseEstimator
-from werkzeug.exceptions import NotFound
+from werkzeug.exceptions import NotFound, UnprocessableEntity, InternalServerError
 
 from gordo import serializer
 
+from .properties import get_tags, get_target_tags
+
 """
 Tools used between different views
 
@@ -33,6 +36,13 @@ basic dataframe output, respectively.
 
 logger = logging.getLogger(__name__)
 
+gordo_name_re = re.compile(r"^[a-zA-Z\d-]+")
+revision_re = re.compile(r"^\d+$")
+
+
+def validate_revision(revision: str) -> bool:
+    return bool(revision_re.match(revision))
+
 
 def dataframe_into_parquet_bytes(
     df: pd.DataFrame, compression: str = "snappy"
@@ -269,7 +279,7 @@ def extract_X_y(method):
     """
 
     @functools.wraps(method)
-    def wrapper_method(self, *args, **kwargs):
+    def wrapper_method(*args, **kwargs):
         start_time = timeit.default_timer()
         # Data provided by the client
         if request.method == "POST":
@@ -290,11 +300,11 @@ def extract_X_y(method):
                 if y is not None:
                     y = dataframe_from_parquet_bytes(y.read())
 
-            X = _verify_dataframe(X, [t.name for t in self.tags])
+            X = _verify_dataframe(X, [t.name for t in get_tags()])
 
             # Verify y if it's not None
             if y is not None:
-                y = _verify_dataframe(y, [t.name for t in self.target_tags])
+                y = _verify_dataframe(y, [t.name for t in get_target_tags()])
 
             # If either X or y came back as a Response type, there was an error
             for data_or_resp in [X, y]:
@@ -315,7 +325,7 @@ def extract_X_y(method):
         logger.debug(f"Time to parse X and y: {timeit.default_timer() - start_time}s")
 
         # And run the original method.
-        return method(self, *args, **kwargs)
+        return method(*args, **kwargs)
 
     return wrapper_method
 
@@ -343,6 +353,16 @@ def load_model(directory: str, name: str) -> BaseEstimator:
     return model
 
 
+def check_metadata_file(directory: str, name: str):
+    """
+    Checking if the directory with metadata exists since it might be deleted through DELETE endpoint
+    """
+    # TODO consider using https://pypi.org/project/ring/ with the ability to delete items from lru cache
+    full_model_dir = os.path.join(directory, name)
+    if not serializer.metadata_path(full_model_dir):
+        raise FileNotFoundError("Unable to load metadata.json file")
+
+
 def load_metadata(directory: str, name: str) -> dict:
     """
     Load metadata from a directory for a given model by name.
@@ -363,7 +383,10 @@ def load_metadata(directory: str, name: str) -> dict:
     return pickle.loads(zlib.decompress(compressed_metadata))
 
 
-@lru_cache(maxsize=25000)
+_n_cached_metadata = int(os.getenv("N_CACHED_METADATA", 250))
+
+
+@lru_cache(maxsize=_n_cached_metadata)
 def _load_compressed_metadata(directory: str, name: str):
     """
     Loads the metadata for model 'name' from directory 'directory', and returns it as a
@@ -379,6 +402,41 @@ def _load_compressed_metadata(directory: str, name: str):
     return zlib.compress(pickle.dumps(metadata))
 
 
+def delete_revision(directory: str, name: str):
+    """
+    Delete model revision
+
+    Parameters
+    ----------
+    directory - Revision directory
+    name - Model name
+    """
+    full_path = os.path.join(directory, name)
+    if not os.path.isfile(os.path.join(full_path, "metadata.json")):
+        raise NotFound("Not found")
+    shutil.rmtree(full_path, ignore_errors=True)
+    if os.path.exists(full_path):
+        raise InternalServerError("Unable to delete this model revision folder")
+    if not os.listdir(directory):
+        shutil.rmtree(directory, ignore_errors=True)
+        if os.path.exists(directory):
+            raise InternalServerError("Unable to delete this revision folder")
+
+
+def validate_gordo_name(gordo_name: str):
+    """
+    gordo_name argument should contains alpha-numericals or '-' symbols
+    """
+    if gordo_name and not gordo_name_re.match(gordo_name):
+        raise UnprocessableEntity("gordo_name field has wrong format")
+
+
+@lru_cache(maxsize=_n_cached_metadata)
+def load_info(directory: str, name: str) -> dict:
+    # TODO better docstring
+    return serializer.load_info(os.path.join(directory, name))
+
+
 def metadata_required(f):
     """
     Decorate a view which has ``gordo_name`` as a url parameter and will
@@ -387,10 +445,17 @@ def metadata_required(f):
 
     @wraps(f)
     def wrapper(*args: tuple, gordo_project: str, gordo_name: str, **kwargs: dict):
+        validate_gordo_name(gordo_name)
+        g.info = {}
         try:
+            g.info = load_info(directory=g.collection_dir, name=gordo_name)
+        except FileNotFoundError:
+            pass
+        try:
+            check_metadata_file(g.collection_dir, gordo_name)
             g.metadata = load_metadata(directory=g.collection_dir, name=gordo_name)
         except FileNotFoundError:
-            raise NotFound(f"No model found for '{gordo_name}'")
+            raise NotFound(f"No metadata found for '{gordo_name}'")
         else:
             return f(*args, **kwargs)
 
@@ -406,7 +471,9 @@ def model_required(f):
 
     @wraps(f)
     def wrapper(*args: tuple, gordo_project: str, gordo_name: str, **kwargs: dict):
+        validate_gordo_name(gordo_name)
         try:
+            check_metadata_file(g.collection_dir, gordo_name)
             g.model = load_model(directory=g.collection_dir, name=gordo_name)
         except FileNotFoundError:
             raise NotFound(f"No such model found: '{gordo_name}'")
@@ -417,33 +484,3 @@ def model_required(f):
             )
 
     return wrapper
-
-
-def find_path_in_dict(path: List[str], data: dict) -> Any:
-    """
-    Find a path in `dict` recursively
-
-    Examples
-    --------
-    >>> find_path_in_dict(["parent", "child"], {"parent": {"child": 42}})
-    42
-
-    Parameters
-    ----------
-    path: List[str]
-    data: dict
-
-    Returns
-    -------
-
-    """
-    reversed_path = copy.copy(path)
-    reversed_path.reverse()
-    curr_data = data
-    while len(reversed_path):
-        key = reversed_path.pop()
-        if key not in curr_data:
-            exception_path = ".".join(path[: len(path) - len(reversed_path)])
-            raise KeyError("'%s' is absent" % exception_path)
-        curr_data = curr_data[key]
-    return curr_data
